var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from '@angular/core';
import { Actions, Effect } from '@ngrx/effects';
import { Observable } from 'rxjs';
import { catchError, map, switchMap } from 'rxjs/operators';
import { SpaceQuery } from '../models/space';
import * as CustomQueryActions from './../actions/custom-query.actions';
import { CustomQueryService } from './../services/custom-query.service';
import { ErrorHandler, filterTypeWithSpace } from './work-item-utils';
var CustomQueryEffects = /** @class */ (function () {
    function CustomQueryEffects(actions$, customQueryService, spaceQuery, errHandler) {
        var _this = this;
        this.actions$ = actions$;
        this.customQueryService = customQueryService;
        this.spaceQuery = spaceQuery;
        this.errHandler = errHandler;
        this.GetCustomQueries$ = this.actions$
            .pipe(filterTypeWithSpace(CustomQueryActions.GET, this.spaceQuery.getCurrentSpace), switchMap(function (_a) {
            var action = _a[0], space = _a[1];
            return _this.customQueryService.getCustomQueries(space.links.self + '/queries')
                .pipe(map(function (types) {
                types = types.map(function (t) {
                    t['selected'] = false;
                    return t;
                });
                return new CustomQueryActions.GetSuccess(types);
            }), catchError(function (err) { return _this.errHandler.handleError(err, 'Problem in fetching custom queries.', new CustomQueryActions.GetError()); }));
        }));
        this.addCustomQuery$ = this.actions$
            .pipe(filterTypeWithSpace(CustomQueryActions.ADD, this.spaceQuery.getCurrentSpace), switchMap(function (_a) {
            var action = _a[0], space = _a[1];
            var payload = action.payload;
            return _this.customQueryService.create(payload, space.links.self + '/queries')
                .pipe(map(function (customQuery) {
                customQuery['selected'] = true;
                var customQueryName = customQuery.attributes.title;
                if (customQueryName.length > 15) {
                    customQueryName = customQueryName.slice(0, 15) + '...';
                }
                return new CustomQueryActions.AddSuccess(customQuery);
            }), catchError(function (err) { return _this.errHandler.handleError(err, 'There was some problem creating custom query.', new CustomQueryActions.AddError()); }));
        }));
        this.deleteCustomQuery = this.actions$
            .pipe(filterTypeWithSpace(CustomQueryActions.DELETE, this.spaceQuery.getCurrentSpace), switchMap(function (_a) {
            var action = _a[0], space = _a[1];
            return _this.customQueryService.delete(space.links.self + '/queries/' + action.payload.id)
                .pipe(map(function () {
                return new CustomQueryActions.DeleteSuccess(action.payload);
            }), catchError(function (err) { return _this.errHandler.handleError(err, 'There was some problem deleting custom query.', new CustomQueryActions.DeleteError()); }));
        }));
    }
    CustomQueryEffects.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    CustomQueryEffects.ctorParameters = function () { return [
        { type: Actions, },
        { type: CustomQueryService, },
        { type: SpaceQuery, },
        { type: ErrorHandler, },
    ]; };
    __decorate([
        Effect(),
        __metadata("design:type", Observable)
    ], CustomQueryEffects.prototype, "GetCustomQueries$", void 0);
    __decorate([
        Effect(),
        __metadata("design:type", Object)
    ], CustomQueryEffects.prototype, "addCustomQuery$", void 0);
    __decorate([
        Effect(),
        __metadata("design:type", Object)
    ], CustomQueryEffects.prototype, "deleteCustomQuery", void 0);
    return CustomQueryEffects;
}());
export { CustomQueryEffects };
//# sourceMappingURL=custom-query.effects.js.map