import { HttpBackend, HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';
export declare class HttpClientService {
    private http;
    constructor(http: HttpClient);
    private setHeaders;
    private requestRetryLogic;
    get<T>(url: string, options?: {}): Observable<T>;
    post<T>(url: string, body: any, options?: any): Observable<T>;
    patch<T>(url: string, body: any, options?: {}): Observable<T>;
    delete(url: string): Observable<any>;
}
/**
 * This is to bypass all interceptors
 * (mainly for detailPage on firefox to solve the redirection problem)
 *
 * Working of HttpHandler, HttpBackend according to Angular code documentation :
 *
 * HttpHandler Transforms an `HttpRequest` into a stream of `HttpEvent`s, one of which will likely be a
 * `HttpResponse`.
 * `HttpHandler` is injectable. When injected, the handler instance dispatches requests to the
 * first interceptor in the chain, which dispatches to the second, etc, eventually reaching the
 * `HttpBackend`.
 * In an `HttpInterceptor`, the `HttpHandler` parameter is the next interceptor in the chain.
 *
 * HttpBackend is the final `HttpHandler` which will dispatch the request via browser HTTP APIs to a backend
 *
 * When injected, `HttpBackend` dispatches requests directly to the backend, without going
 * through the interceptor chain.
 *
 */
export declare class HttpBackendClient extends HttpClient {
    constructor(handler: HttpBackend);
}
